﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using AIOWDMNet;

namespace DIOSampleAIOWDM
{
    public partial class InputSampleForm : Form
    {
        private uint cardNum;
        private static readonly uint[] byteIndexToRegiserOffset = new uint[] { 0x00, 0x01, 0x04, 0x05 };
        private void DIO_Write8(uint cardNum, uint byteIndex, byte value) { AIOWDM.RelOutPortB( cardNum, byteIndexToRegiserOffset[byteIndex ], value); }
        private uint DIO_Read8(uint cardNum, uint byteIndex) { return AIOWDM.RelInPortB( cardNum, byteIndexToRegiserOffset[byteIndex]); }
        private byte DIO_Write1(uint cardNum, uint bitIndex, bool value)
        {
            /* read byte value from port */
            uint data = DIO_Read8(cardNum, bitIndex / 8);  // TODO: check for 0xAA55 and error if found

            /* perform binary arithmetic to merge new bit appropriately */
            byte mask = (byte)((uint)1 << (int)(bitIndex % 8));
            if (value == true)
                data |= mask;
            else
                data &= ~(uint)mask;

            /* finish the read-modify-write operation by writing the new data value to the byte of outputs */
            AIOWDM.RelOutPortB( cardNum, byteIndexToRegiserOffset[bitIndex / 8], (byte) data);
            return 0;
        }

        public struct Info {
            public Info(byte byts, byte bits, string type) { numDIOBytes = byts; numDIOBits = bits; typ = type; }
            public  byte numDIOBytes;
            public byte numDIOBits;
            public readonly string typ;
        }

        private Dictionary<uint, Info> validDeviceIDLookup = new Dictionary<uint, Info>();
        public InputSampleForm()
        {
            InitializeComponent();
            validDeviceIDLookup.Add(0x0f00, new Info(2, 8, "Relays"));   // PCI-IIRO-8
            validDeviceIDLookup.Add(0x0f01, new Info(2, 8, "Relays"));   // LPCI-IIRO-8
            validDeviceIDLookup.Add(0x0f02, new Info(2, 8, "Relays"));   // PCIe-IIRO-8
            validDeviceIDLookup.Add(0x0101, new Info(2, 8, "FETs"));     // mPCIe-IDIO-8
            validDeviceIDLookup.Add(0x0102, new Info(2, 8, "Relays"));   // mPCIe-IIRO-8
            validDeviceIDLookup.Add(0x0103, new Info(2, 4, "FETs"));     // mPCIe-IDIO-4
            validDeviceIDLookup.Add(0x0104, new Info(2, 4, "Relays"));   // mPCIe-IIRO-4
            validDeviceIDLookup.Add(0x0110, new Info(2, 8, "FETs"));     // mPCIe-IDIO-8HL
            validDeviceIDLookup.Add(0x0111, new Info(2, 8, "FETs"));     // mPCIe-IDIO-8H
            validDeviceIDLookup.Add(0x0112, new Info(2, 8, "FETs"));     // mPCIe-IDIO-8L
        }

        private Dictionary<uint, Info> foundDeviceData = new Dictionary<uint, Info>();
        private int pollCount;

        private void InputSampleForm_Load(object sender, EventArgs e)
        {
            var numCards = AIOWDM.GetNumCards();
            log.Items.Add("AIOWDM.GetNumCards reports " + numCards + " cards installed");

            for (int numCard = 0; numCard < numCards; numCard++)
            {
                var Status = AIOWDM.QueryCardInfo(numCard, out uint deviceID, out uint baseAddress, out string cardName);
                if (Status == 0)
                    log.Items.Add("AIOWDM.QueryCardInfo returned FALSE - Error for Card Number " + numCard);
                else
                {
                    cardList.Items.Add("CardNum #" + numCard.ToString("d2") + ": " + cardName);
                    foundDeviceData.Add((uint)(numCard), validDeviceIDLookup[deviceID]);
                    cardList.SelectedIndex = 0;
                }
            }
        }

        private void CardList_SelectedIndexChanged(object sender, EventArgs e)
        {
            cardNum = (uint )cardList.SelectedIndex;
            DigitalInputPollTimer.Enabled = true;

            var dataReadback = DIO_Read8(cardNum, 0);
            out0.Checked = (dataReadback & 0x01) != 0;
            out1.Checked = (dataReadback & 0x02) != 0;
            out2.Checked = (dataReadback & 0x04) != 0;
            out3.Checked = (dataReadback & 0x08) != 0;
            out4.Checked = (dataReadback & 0x10) != 0;
            out5.Checked = (dataReadback & 0x20) != 0;
            out6.Checked = (dataReadback & 0x40) != 0;
            out7.Checked = (dataReadback & 0x80) != 0;
        }

        private void DigitalInputPollTimer_Tick(object sender, EventArgs e)
        {
            statusEdit.Text = "poll # " + ++pollCount;

            var dataReadback = DIO_Read8(cardNum, 0);
            var dataIsoInput = DIO_Read8(cardNum, 1);
            ISOReadback.Text = Convert.ToString(dataReadback, 2).PadLeft(8, '0') + " " + foundDeviceData[cardNum].typ + " Readback";
            ISOInput.Text = Convert.ToString(dataIsoInput, 2).PadLeft(8, '0') + " Iso In";
        }

        private void Outputs_Click(object sender, EventArgs e)
        {
            var box = sender as CheckBox;
            byte bit = byte.Parse(box.Tag as string);
            var active = box.Checked;
            DIO_Write1(cardNum, bit, active);
        }

        private void out0_CheckedChanged(object sender, EventArgs e)
        {

        }
    }
}
