﻿using AIOWDMNet;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace DIOSampleAIOWDM
{
    public partial class InputSampleForm : Form
    {
        private uint cardNum;
        private static readonly uint[] byteIndexToRegiserOffset = new uint[] { 0x10 };
        private void DIO_Write8(uint cardNum, uint byteIndex, byte value) { AIOWDM.RelOutPortB( cardNum, byteIndexToRegiserOffset[byteIndex ], value); }
        private uint TTL_Read8(uint cardNum, uint byteIndex) { return AIOWDM.RelInPortB( cardNum, byteIndexToRegiserOffset[byteIndex]); }
        private byte DIO_Write1(uint cardNum, UInt32 bitIndex, bool value)
        {
            /* read byte value from port */
            uint data = TTL_Read8(cardNum, bitIndex / 8);  // TODO: check for 0xAA55 and error if found

            /* perform binary arithmetic to merge new bit appropriately */
            byte mask = (byte)((uint)1 << (int)(bitIndex % 8));
            if (value == true)
                data |= mask;
            else
                data &= ~(uint)mask;

            /* finish the read-modify-write operation by writing the new data value to the byte of outputs */
            AIOWDM.RelOutPortB(cardNum, byteIndexToRegiserOffset[bitIndex / 8], (byte) data);
            return 0;
        }
        
        public struct Info {
            public Info(byte byts, byte bits, string type) { numDIOBytes = byts; numDIOBits = bits; typ = type; }
            public  byte numDIOBytes;
            public byte numDIOBits;
            public readonly string typ;
        }

        private Dictionary<uint, Info> validDeviceIDLookup = new Dictionary<uint, Info>();
        public InputSampleForm()
        {
            InitializeComponent();
            validDeviceIDLookup.Add(0x0101, new Info(2, 8, "TTL") );     // mPCIe-IDIO-8
            validDeviceIDLookup.Add(0x0103, new Info(2, 4, "TTL") );     // mPCIe-IDIO-4
            validDeviceIDLookup.Add(0x0110, new Info(2, 8, "TTL") );     // mPCIe-IDIO-8HL
            validDeviceIDLookup.Add(0x0111, new Info(2, 8, "TTL") );     // mPCIe-IDIO-8H
            validDeviceIDLookup.Add(0x0112, new Info(2, 8, "TTL") );     // mPCIe-IDIO-8L
        }

        private Dictionary<uint, Info> foundDeviceData = new Dictionary<uint, Info>();
        private int pollCount;
        private byte IOMask;

        private void InputSampleForm_Load(object sender, EventArgs e)
        {
            var numCards = AIOWDM.GetNumCards();
            log.Items.Add("AIOWDM.GetNumCards reports " + numCards + " cards installed");
            if (numCards == 0) {log.Items.Add("Please confirm your device is installed and appears without errors in Device Manager."); }
            for (uint numCard = 0; numCard < numCards; numCard++)
            {
                var Status = AIOWDM.QueryCardInfo(numCard, out uint deviceID, out uint _, out string cardName);
                if (Status == 0)
                    log.Items.Add("AIOWDM.QueryCardInfo returned FALSE (Error) for Card Number " + numCard);
                else
                {
                    if (!validDeviceIDLookup.ContainsKey(deviceID))
                    {
                        cardName = cardName.TrimEnd('\0');
                        cardList.Items.Add("CardNum #" + numCard.ToString("d2") + ": " + cardName + " — NOT for this sample");
                    }
                    else
                    {
                        cardList.SelectedIndex = cardList.Items.Add("CardNum #" + numCard.ToString("d2") + ": " + cardName);
                        foundDeviceData.Add(numCard, validDeviceIDLookup[deviceID]);
                    }
                }
            }
        }

        private void CardList_SelectedIndexChanged(object sender, EventArgs e)
        {
            cardNum = (uint)cardList.SelectedIndex;
            DigitalInputPollTimer.Enabled = true;
            ReadbackOutputs();
        }

        private void ReadbackOutputs()
        {
            var data = TTL_Read8(cardNum, 0);
            bit0.Checked = (data & 0x01) != 0;
            bit1.Checked = (data & 0x02) != 0;
            bit2.Checked = (data & 0x04) != 0;
            bit3.Checked = (data & 0x08) != 0;
            bit4.Checked = (data & 0x10) != 0;
            bit5.Checked = (data & 0x20) != 0;
            bit6.Checked = (data & 0x40) != 0;
            bit7.Checked = (data & 0x80) != 0;
        }

        private void DigitalInputPollTimer_Tick(object sender, EventArgs e)
        {
            statusEdit.Text = "poll # " + ++pollCount;

            var data = TTL_Read8(cardNum, 0);

            IOGroup0.Text = Convert.ToString(data & 0x0F, 2).PadLeft(4, '0') + " TTL #3 - #0 read";
            IOGroup1.Text = Convert.ToString(data >> 4 & 0x0f, 2).PadLeft(4, '0') + " TTL #7 - #4 read";
        }

        private void Outputs_Click(object sender, EventArgs e)
        {
        }

        private void CbIO_CheckedChanged(object sender, EventArgs e)
        {
            if ((sender as CheckBox).Checked)
            {
                IOMask =(byte)( IOMask | (1 << Convert.ToInt16((sender as CheckBox).Tag)) );
            }
            else
                IOMask = (byte)(IOMask & ~ (1 << Convert.ToInt16((sender as CheckBox).Tag)));

            AIOWDM.RelOutPortB(cardNum, 0x11, IOMask);

            if (Convert.ToInt16((sender as CheckBox).Tag) == 0) OutPanel03.Enabled = (sender as CheckBox).Checked;
            else OutPanel47.Enabled = (sender as CheckBox).Checked;
            ReadbackOutputs();
        }
        
        private void WriteOutputBit(object sender, EventArgs e)
        {
            DIO_Write1(cardNum, Convert.ToUInt32((sender as CheckBox).Tag), (sender as CheckBox).Checked);
        }
    }
}
